import path from 'path';
import { wipe } from './wipeCache';
import { _clearPlugins } from './plugins';
import plugins from './plugins/index';
import { getModuleName, getModuleParent } from './module';
import { setExtensions as resolveExtensions } from './constants';

var moduleName = getModuleName(module);
if (!moduleName) {
  throw new Error('Rewiremock: there is no "module name". If you are using Jest - disable automocking.');
}

if (!getModuleParent(module)) {
  throw new Error('Rewiremock: there is no "parent module". Is there two HotModuleReplacementPlugins?');
}

function safelyRemoveCache(moduleName) {
  var m = require.cache[moduleName];
  if (m) {
    if (m.parent && m.parent.children) {
      m.parent.children = m.parent.children.filter(function (x) {
        return x !== m;
      });
    }
    delete require.cache[moduleName];
  }
}

// delete core
safelyRemoveCache(path.join(path.dirname(__filename), './mockModule.js'));
// delete self
safelyRemoveCache(moduleName.replace('index.js', 'mockModule.js'));

import * as API from './mockModule';
import applyDefaultConfig from "./plugins/defaultConfig";

export var cleanup = function cleanup() {
  var wipeAll = function wipeAll(stubs, moduleName) {
    return moduleName.indexOf(stubs) === 0;
  };
  wipe(path.dirname(__filename), wipeAll);
};

export var overrideEntryPoint = function overrideEntryPoint(module) {
  safelyRemoveCache(getModuleName(module));
  API.mockModule.overrideEntryPoint(getModuleParent(module));
  //API.cleanup();
};

overrideEntryPoint(module);

// instance must be clean
API.mockModule.clear();
_clearPlugins();

var addPlugin = API.addPlugin;
var removePlugin = API.removePlugin;

//addPlugin(plugins.nodejs);

addPlugin(plugins.toBeUsed);
addPlugin(plugins.directChild);

addPlugin(plugins.__mock__);

if (typeof __webpack_require__ !== "undefined") {
  addPlugin(plugins.nodeLibBrowser);
}

applyDefaultConfig(API.mockModule);

if (global['_REWIREMOCK_HOISTED_']) {
  global['_REWIREMOCK_HOISTED_'].forEach(function (cb) {
    cb(API.mockModule, { plugins: plugins, overrideEntryPoint: overrideEntryPoint });
  });
  global['_REWIREMOCK_HOISTED_'] = [];
}

export { addPlugin, removePlugin, plugins, resolveExtensions };

export default API.mockModule;